/* ipt_ACCRATE.c Account rate limting module */

/* Copyright (c) 2011, TeamF1, Inc. */

/* modification history:
 * ---------------------
 * 01a, 25jul11,phb created
 */

/* Shared library add-on to iptables to add CLASSIFY target support. */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include <errno.h>

#include <xtables.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter_ipv4/ipt_ACCRATE.h>
#include <linux/types.h>
#include <linux/pkt_sched.h>

/* Function which prints out usage message. */
static void
help(void)
{
	printf(
"ACCRATE target v%s options:\n"
"  --dir direction --limit limit in MB --file file name\n"
"\n",
XTABLES_VERSION);
}

static struct option opts[] = {
	{ "dir", 1, 0, '1' },
    { "limit", 1, 0, '2'},
    { "file", 1, 0, '3'},
	{ 0 }
};

#define IPT_ACCRATE_DIR     0x01
#define IPT_ACCRATE_LIMIT   0x02
#define IPT_ACCRATE_FILE     0x04

/* Function which parses command options; returns true if it
   ate an option */
static int
parse(int c, char **argv, int invert, unsigned int *flags,
      const void *entry,
      struct xt_entry_target **target)
{
	struct ipt_accrate_target_info *acinfo
		= (struct ipt_accrate_target_info *)(*target)->data;

	switch (c) {
	case '1':
	    {
        if (*flags & IPT_ACCRATE_DIR)
			xtables_error(PARAMETER_PROBLEM,
			           "ACCRATE: Can't specify dir twice");

        if (strncmp(optarg, "both", strlen(optarg)) == 0)
            acinfo->dir = 1;
        else if (strncmp(optarg, "down", strlen(optarg)) == 0)
            acinfo->dir = 0;
        else
            xtables_error(PARAMETER_PROBLEM,
			           "ACCRATE: Invalid direction name %s ", optarg);
		
		*flags |= IPT_ACCRATE_DIR;
		break;
        }
    case '2':
	    {
        if (*flags & IPT_ACCRATE_LIMIT)
			xtables_error(PARAMETER_PROBLEM,
			           "ACCRATE: Can't specify limit twice");
        if (!xtables_strtoui(optarg, NULL,
                             (unsigned int *)&acinfo->limit, 0, 0))
            xtables_error(PARAMETER_PROBLEM,
			           "ACCRATE: Bad limit value : %s ", optarg);
		
		*flags |= IPT_ACCRATE_LIMIT;
		break;
        }
    case '3':
	    {
        if (*flags & IPT_ACCRATE_FILE)
			xtables_error(PARAMETER_PROBLEM,
			           "ACCRATE: Can't specify file twice");
        strncpy(acinfo->file, optarg, IPT_ACCRATE_NAME_SIZE);
		
		*flags |= IPT_ACCRATE_FILE;
		break;
        }
	default:
		return 0;
	}

	return 1;
}

static void
final_check(unsigned int flags)
{
	if (!flags)
		xtables_error(PARAMETER_PROBLEM,
		           "ACCRATE: Parameters --dir --limit --file are required");
}


/* Prints out the targinfo. */
static void
print(const void *ip,
      const struct xt_entry_target *target,
      int numeric)
{
	const struct ipt_accrate_target_info *clinfo =
		(const struct ipt_accrate_target_info *)target->data;

	printf("--dir %s  --limit %d  --file %s ",
	       (clinfo->dir)? "BOTH":"DOWNLOAD", (clinfo->limit), (clinfo->file));
}

/* Saves the union ipt_targinfo in parsable form to stdout. */
static void
save(const void *ip, const struct xt_entry_target *target)
{
	const struct ipt_accrate_target_info *clinfo =
		(const struct ipt_accrate_target_info *)target->data;

	printf("--dir %s  --limit %d  --file %s ",
	       (clinfo->dir)? "BOTH":"DOWNLOAD", (clinfo->limit), (clinfo->file));
}

static struct xtables_target accrate = { 
	.next		= NULL,
	.name		= "ACCRATE",
	.version	= XTABLES_VERSION,
    .family			= NFPROTO_IPV4,
	.size		= XT_ALIGN(sizeof(struct ipt_accrate_target_info)),
	.userspacesize	= XT_ALIGN(sizeof(struct ipt_accrate_target_info)),
	.help		= help,
	.parse		= parse,
	.final_check	= final_check,
	.print		= print,
	.save		= save,
	.extra_opts	= opts
};

void _init(void)
{
	xtables_register_target(&accrate);
}
